==> h-orm-0.0.1-sources.jar: cn/sylinx/horm/dialect/sql/DefaultSqlBuilder.java <==
package cn.sylinx.horm.dialect.sql;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import cn.sylinx.horm.exception.HORMException;
import cn.sylinx.horm.model.anno.PrimaryKey;
import cn.sylinx.horm.model.base.Model;
import cn.sylinx.horm.model.cache.ModelCacheUtil;
import cn.sylinx.horm.model.cache.ModelFabric;
import cn.sylinx.horm.model.optlock.OptimisticLockInsert;
import cn.sylinx.horm.model.optlock.OptimisticLockUpdate;
import cn.sylinx.horm.model.optlock.OptimisticLockWrapper;
import cn.sylinx.horm.util.GLog;
import cn.sylinx.horm.util.Pair;
import cn.sylinx.horm.util.StrKit;
import cn.sylinx.horm.util.Tuple;

class DefaultSqlBuilder implements SqlBuilder {

	private String escape0 = "";
	private String escape1 = "";

	public DefaultSqlBuilder() {
		String[] escapes = getEscapeChar();
		if (escapes != null && escapes.length >= 2) {
			escape0 = escapes[0];
			escape1 = escapes[1];
		}
	}

	protected String[] getEscapeChar() {
		return new String[] { "", "" };
	}

	@Override
	public Tuple buildPaginatorSql(String preSql, int pageNumber, int pageSize) {

		String sqlCount = "select count(1) as totalCount from (" + preSql + ") as temp";
		String sql = preSql + " limit ?, ? ";
		int offset = pageSize * (pageNumber - 1);
		Object[] params = new Object[] { offset, pageSize };
		return Tuple.apply(sqlCount, sql, params);
	}

	protected List<String> getPrimaryKey(final PrimaryKey pks) {
		List<String> pklist = new ArrayList<String>();
		if (pks != null) {
			String[] pkArray = pks.value();
			for (String pk : pkArray) {
				if (StrKit.isNotBlank(pk)) {
					pklist.add(pk);
				}
			}
		}

		if (pklist.isEmpty()) {
			pklist.add("id");
		}
		return pklist;
	}

	@Override
	public <T> Pair buildDeleteSQL(T t) {

		if (t == null) {
			throw new HORMException("model is null");
		}

		ModelFabric modelFabric = ModelCacheUtil.getModelFabric(t.getClass());
		String tableName = modelFabric.getTableName();
		final PrimaryKey pk = modelFabric.getPrimaryKey();
		List<String> pks = getPrimaryKey(pk);
		Map<String, String> map = modelFabric.getAttrMapping();

		final String token = " and ";

		// 获取所有字段
		List<Field> fields = ModelCacheUtil.getObjectAllFieldsWithcache(t.getClass());
		StringBuilder sql = new StringBuilder("delete from " + escape0 + tableName + escape1 + " where ");

		boolean find = false;
		List<Object> params = new ArrayList<Object>();

		try {

			for (Field item : fields) {

				String fieldName = item.getName();

				if (pks.contains(fieldName)) {
					// 字段
					String f = map.get(fieldName);
					item.setAccessible(true);
					Object v = item.get(t);

					if (f != null && v != null) {
						find = true;
						sql.append(escape0).append(f).append(escape1).append(" = ? ").append(token);
						params.add(v);
					}
				}
			}

		} catch (IllegalAccessException e) {
			throw new HORMException(e);
		}

		if (!find) {
			throw new HORMException("primary key has no value");
		}

		int totalLen = sql.length();
		int index = sql.length() - token.length();
		sql.delete(index, totalLen);

		Object[] objects = new Object[params.size()];
		for (int i = 0; i < params.size(); ++i) {
			objects[i] = params.get(i);
		}

		String sqlNative = sql.toString();
		GLog.debug("sql:{} , params:{}", sqlNative, objects);

		return Pair.apply(sqlNative, objects);
	}

	@Override
	public <T> Pair buildInsertSQL(T t) {

		if (t == null) {
			throw new HORMException("model is null");
		}

		ModelFabric mf = ModelCacheUtil.getModelFabric(t.getClass());
		Map<String, String> map = mf.getAttrMapping();
		String tableName = mf.getTableName();

		if (map == null || map.isEmpty()) {
			GLog.error("insert mapper is empty~");
			throw new HORMException("insert mapper is empty~");
		}

		if (StrKit.isBlank(tableName)) {
			GLog.error("table name is empty~");
			throw new HORMException("table name is empty~");
		}

		// 获取所有字段
		List<Field> fields = ModelCacheUtil.getObjectAllFieldsWithcache(t.getClass());

		if (fields.isEmpty()) {
			GLog.error("no fields");
			throw new HORMException("no fields");
		}

		StringBuilder fds = new StringBuilder();
		StringBuilder fdsv = new StringBuilder();
		List<Object> params = new ArrayList<Object>();

		fds.append("(");
		fdsv.append("(");

		OptimisticLockInsert optimisticLockInsert = OptimisticLockWrapper.parseOptimisticLockInsert(t);

		try {

			if (optimisticLockInsert != null) {
				optimisticLockInsert.getVersionField().setAccessible(true);
				optimisticLockInsert.getVersionField().set(t, optimisticLockInsert.getInitVersionValue());
			}

			for (Field item : fields) {
				String fieldName = item.getName();
				if (map.containsKey(fieldName)) {
					item.setAccessible(true);
					Object v = item.get(t);
					// 值不为空才插入
					if (v != null) {
						String jdbcFieldName = map.get(fieldName);
						fds.append(escape0).append(jdbcFieldName).append(escape1).append(",");
						fdsv.append("?,");
						params.add(v);
					}
				}
			}
		} catch (IllegalAccessException e) {
			throw new HORMException(e);
		}

		// 存在插入字段
		if (fds.length() > 1) {

			fds.deleteCharAt(fds.length() - 1);
			fdsv.deleteCharAt(fdsv.length() - 1);
			fds.append(")");
			fdsv.append(")");

		} else {

			GLog.error("no insert fields");
			throw new HORMException("no insert fields");
		}

		StringBuilder sb = new StringBuilder();
		sb.append("insert into ").append(escape0).append(tableName).append(escape1).append(fds).append(" values ")
				.append(fdsv);

		Object[] objects = new Object[params.size()];
		for (int i = 0; i < params.size(); ++i) {
			objects[i] = params.get(i);
		}

		String sql = sb.toString();

		GLog.debug("sql:{} , params:{}", sql, objects);
		return Pair.apply(sql, objects);
	}

	@Override
	public <T> Pair buildUpdateSQL(T t) {

		if (t == null) {
			throw new HORMException("model is null");
		}

		ModelFabric mf = ModelCacheUtil.getModelFabric(t.getClass());
		Map<String, String> map = mf.getAttrMapping();
		List<String> pks = getPrimaryKey(mf.getPrimaryKey());
		String tableName = mf.getTableName();

		if (map == null || map.isEmpty()) {
			GLog.error("update mapper is empty~");
			return null;
		}

		if (StrKit.isBlank(tableName)) {
			GLog.error("table name is empty~");
			return null;
		}

		if (pks == null) {
			pks = Collections.emptyList();
		}

		// 获取所有字段
		List<Field> fields = ModelCacheUtil.getObjectAllFieldsWithcache(t.getClass());

		if (fields.isEmpty()) {
			return null;
		}

		// 主键
		Map<String, Object> pkKv = new HashMap<String, Object>();

		StringBuilder fds = new StringBuilder();
		List<Object> params = new ArrayList<Object>();

		// 获取可设置空字段值
		Set<String> nullableSets = new HashSet<String>();
		if (t instanceof Model) {
			nullableSets = ((Model) t).getNullableFields();
		}

		// 乐观锁信息
		OptimisticLockUpdate olu = OptimisticLockWrapper.parseOptimisticLockUpdate(t);

		try {

			if (olu != null) {
				olu.getVersionField().setAccessible(true);
				olu.getVersionField().set(t, olu.getNewVersionValue());
			}

			for (Field item : fields) {

				item.setAccessible(true);
				String fieldName = item.getName();

				// 判断主键
				if (pks.contains(fieldName)) {

					Object pkValue = item.get(t);
					String columnKey = map.get(fieldName);
					pkKv.put(columnKey, pkValue);

				} else {
					// 字段
					String f = map.get(fieldName);
					Object v = item.get(t);

					if (f != null) {// 找到字段

						boolean nullable = nullableSets.contains(fieldName);// 可为空

						if (v != null) {
							fds.append(escape0).append(f).append(escape1).append(" = ?,");
							params.add(v);
						} else if (nullable) {
							// 可为空，则设置null值
							fds.append(escape0).append(f).append(escape1).append(" = NULL,");
						}

					}

				}
			}
		} catch (IllegalAccessException e) {
			throw new HORMException(e);
		}

		// 存在插入字段
		if (fds.length() > 0) {

			fds.deleteCharAt(fds.length() - 1);

		} else {

			throw new RuntimeException("update values is empty ~");
		}

		StringBuilder sb = new StringBuilder();
		sb.append("update ").append(escape0).append(tableName).append(escape1).append(" set ").append(fds);

		if (!pkKv.isEmpty()) {

			final String token = " and ";

			sb.append(" where ");

			if (pks.size() == pkKv.size()) {
				// 全部匹配主键，按顺序组合

				for (String k : pks) {
					String c = map.get(k);
					sb.append(escape0).append(c).append(escape1).append(" = ?").append(token);
					params.add(pkKv.get(c));
				}

			} else {

				for (Entry<String, Object> entry : pkKv.entrySet()) {
					sb.append(escape0).append(entry.getKey()).append(escape1).append(" = ?").append(token);
					params.add(entry.getValue());
				}
			}

			// 此处加入乐观锁
			if (olu != null) {
				String columnKey = map.get(olu.getVersionFieldAttr());
				sb.append(escape0).append(columnKey).append(escape1).append(" = ?").append(token);
				params.add(olu.getOldVersionValue());
			}

			int totalLen = sb.length();
			int index = sb.length() - token.length();
			sb.delete(index, totalLen);
		}

		Object[] objects = new Object[params.size()];
		for (int i = 0; i < params.size(); ++i) {
			objects[i] = params.get(i);
		}

		String sql = sb.toString();
		GLog.debug("sql:{} , params:{}", sql, objects);

		return Pair.apply(sql, objects);
	}

	@Override
	public Pair buildDeleteByFieldSQL(List<Pair> kvList, Class<?> clz) {

		ModelFabric mf = ModelCacheUtil.getModelFabric(clz);

		String table = mf.getTableName();
		if (table == null) {
			throw new HORMException("对象没有Table注解");
		}

		Map<String, String> am = mf.getAttrMapping();
		if (am == null) {
			throw new HORMException("对象映射字段缺失");
		}

		String andToken = " and ";
		StringBuilder sb = new StringBuilder();
		List<Object> paramList = new ArrayList<>();

		for (int i = 0; i < kvList.size(); ++i) {
			Pair kv = kvList.get(i);
			String field = kv.getKey();
			Object value = kv.getValue();
			String trueField = am.get(field);
			if (trueField == null) {
				trueField = field;
			}
			if (value == null) {
				sb.append(escape0).append(trueField).append(escape1).append(" is null").append(andToken);
			} else {
				sb.append(escape0).append(trueField).append(escape1).append(" = ?").append(andToken);
				paramList.add(value);
			}

		}
		sb.delete(sb.length() - andToken.length(), sb.length());
		String sql = "delete from " + escape0 + table + escape1 + " where " + sb.toString();

		return Pair.apply(sql, paramList.toArray());
	}

	@Override
	public Pair buildQueryByFieldSQL(List<Pair> kvList, Class<?> clz) {

		return buildQueryByFieldSQL(kvList, clz, null);
	}

	@Override
	public Pair buildQueryByFieldSQL(List<Pair> kvList, Class<?> clz, String[] fields) {

		ModelFabric mf = ModelCacheUtil.getModelFabric(clz);

		String table = mf.getTableName();
		if (table == null) {
			throw new HORMException("对象没有Table注解");
		}

		Map<String, String> am = mf.getAttrMapping();
		if (am == null) {
			throw new HORMException("对象映射字段缺失");
		}

		String andToken = " and ";
		StringBuilder sb = new StringBuilder();

		List<Object> paramList = new ArrayList<>();

		for (int i = 0; i < kvList.size(); ++i) {
			Pair kv = kvList.get(i);

			String field = kv.getKey();
			Object value = kv.getValue();

			String trueField = am.get(field);
			if (trueField == null) {
				trueField = field;
			}
			if (value == null) {
				sb.append(escape0).append(trueField).append(escape1).append(" is null").append(andToken);
			} else {
				sb.append(escape0).append(trueField).append(escape1).append(" = ?").append(andToken);
				paramList.add(value);
			}

		}
		sb.delete(sb.length() - andToken.length(), sb.length());

		// 选择字段
		StringBuilder sf = new StringBuilder();
		if (fields != null && fields.length > 0) {
			for (String field : fields) {
				String trueField = am.get(field);
				if (trueField == null) {
					trueField = field;
				}
				sf.append(escape0).append(trueField).append(escape1).append(",");
			}
			sf.deleteCharAt(sf.length() - 1);
		} else {
			sf.append("*");
		}

		String sql = "select " + sf.toString() + " from " + escape0 + table + escape1 + " where " + sb.toString();

		return Pair.apply(sql, paramList.toArray());
	}

	@Override
	public String buildSimpleQuery(Class<?> clz) {
		return buildSimpleQuery(clz, null);
	}

	@Override
	public String buildSimpleQuery(Class<?> clz, String[] fields) {

		ModelFabric mf = ModelCacheUtil.getModelFabric(clz);
		String table = mf.getTableName();
		if (table == null) {
			throw new RuntimeException("对象没有Table注解");
		}

		Map<String, String> am = mf.getAttrMapping();
		if (fields == null || fields.length == 0 || am == null) {
			return "select * from " + escape0 + table + escape1;
		}

		StringBuilder sb = new StringBuilder("select ");
		for (String field : fields) {
			String trueField = am.get(field);
			if (trueField == null) {
				trueField = field;
			}
			sb.append(escape0).append(trueField).append(escape1).append(",");
		}
		sb.deleteCharAt(sb.length() - 1);
		sb.append(" from ").append(escape0).append(table).append(escape1);
		return sb.toString();
	}

	@Override
	public String buildValidateQuery() {
		return "SELECT 1";
	}

	@Override
	public String getTable(Class<?> clz) {
		return ModelCacheUtil.getModelFabric(clz).getTableName();
	}

	@Override
	public String getTableColumn(Class<?> clz, String prop) {
		ModelFabric mf = ModelCacheUtil.getModelFabric(clz);
		Map<String, String> attrMap = mf.getAttrMapping();
		String column = attrMap.get(prop);
		if (column == null) {
			column = prop;
		}
		return column;
	}

	@Override
	public String[] buildCreateTableDDL(Class<?> clz) {
		throw new UnsupportedOperationException("not implement");
	}

	@Override
	public String[] buildModifyTableColumnDDL(Class<?> clz, String prop) {
		throw new UnsupportedOperationException("not implement");
	}

	@Override
	public String[] buildAddTableColumnDDL(Class<?> clz, String prop) {
		throw new UnsupportedOperationException("not implement");
	}

}

==> h-orm-0.0.5-sources.jar: cn/sylinx/horm/dialect/sql/DefaultSqlBuilder.java <==
package cn.sylinx.horm.dialect.sql;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import cn.sylinx.horm.config.OrmConfigHolder;
import cn.sylinx.horm.exception.HORMException;
import cn.sylinx.horm.model.anno.PrimaryKey;
import cn.sylinx.horm.model.base.BaseModel;
import cn.sylinx.horm.model.base.Model;
import cn.sylinx.horm.model.cache.ModelCacheUtil;
import cn.sylinx.horm.model.cache.ModelFabric;
import cn.sylinx.horm.model.optlock.OptimisticLockInsert;
import cn.sylinx.horm.model.optlock.OptimisticLockUpdate;
import cn.sylinx.horm.model.optlock.OptimisticLockWrapper;
import cn.sylinx.horm.util.GLog;
import cn.sylinx.horm.util.Pair;
import cn.sylinx.horm.util.StrKit;
import cn.sylinx.horm.util.Tuple;

public class DefaultSqlBuilder implements SqlBuilder {

	private String escape0 = "";
	private String escape1 = "";

	public DefaultSqlBuilder() {
		String[] escapes = getEscapeChar();
		if (escapes != null && escapes.length >= 2) {
			escape0 = escapes[0];
			escape1 = escapes[1];
		}
	}

	protected String[] getEscapeChar() {
		return new String[] { "", "" };
	}

	protected Object convertValue(Object v) {
		return v;
	}

	@Override
	public Tuple buildPaginatorSql(String preSql, int pageNumber, int pageSize) {

		String sqlCount = "select count(1) as totalCount from (" + preSql + ") as temp";
		String sql = preSql + " limit ?, ? ";
		int offset = pageSize * (pageNumber - 1);
		Object[] params = new Object[] { offset, pageSize };
		return Tuple.apply(sqlCount, sql, params);
	}

	protected List<String> getPrimaryKey(final PrimaryKey pks) {
		List<String> pklist = new ArrayList<String>();
		if (pks != null) {
			String[] pkArray = pks.value();
			for (String pk : pkArray) {
				if (StrKit.isNotBlank(pk)) {
					pklist.add(pk);
				}
			}
		}

		if (pklist.isEmpty()) {
			pklist.add("id");
		}
		return pklist;
	}

	@Override
	public <T> Pair buildDeleteSQL(T t) {

		if (t == null) {
			throw new HORMException("model is null");
		}

		ModelFabric modelFabric = ModelCacheUtil.getModelFabric(t.getClass());
		String tableName = modelFabric.getTableName();
		final PrimaryKey pk = modelFabric.getPrimaryKey();
		List<String> pks = getPrimaryKey(pk);
		Map<String, String> map = modelFabric.getAttrMapping();

		final String token = " and ";

		// 获取所有字段
		List<Field> fields = ModelCacheUtil.getObjectAllFieldsWithcache(t.getClass());
		StringBuilder sql = new StringBuilder("delete from " + escape0 + tableName + escape1 + " where ");

		boolean find = false;
		List<Object> params = new ArrayList<Object>();

		try {

			for (Field item : fields) {

				String fieldName = item.getName();

				if (pks.contains(fieldName)) {
					// 字段
					String f = map.get(fieldName);
					item.setAccessible(true);
					Object v = item.get(t);

					if (f != null && v != null) {
						find = true;
						sql.append(escape0).append(f).append(escape1).append(" = ? ").append(token);
						params.add(convertValue(v));
					}
				}
			}

		} catch (IllegalAccessException e) {
			throw new HORMException(e);
		}

		if (!find) {
			throw new HORMException("primary key has no value");
		}

		int totalLen = sql.length();
		int index = sql.length() - token.length();
		sql.delete(index, totalLen);

		Object[] objects = new Object[params.size()];
		for (int i = 0; i < params.size(); ++i) {
			objects[i] = params.get(i);
		}

		String sqlNative = sql.toString();
		GLog.debug("sql:{} , params:{}", sqlNative, objects);

		return Pair.apply(sqlNative, objects);
	}

	@Override
	public <T> Pair buildInsertSQL(T t) {

		if (t == null) {
			throw new HORMException("model is null");
		}

		ModelFabric mf = ModelCacheUtil.getModelFabric(t.getClass());
		Map<String, String> map = mf.getAttrMapping();
		String tableName = mf.getTableName();

		if (map == null || map.isEmpty()) {
			GLog.error("insert mapper is empty");
			throw new HORMException("insert mapper is empty");
		}

		if (StrKit.isBlank(tableName)) {
			GLog.error("table name is empty");
			throw new HORMException("table name is empty");
		}

		// 获取所有字段
		List<Field> fields = ModelCacheUtil.getObjectAllFieldsWithcache(t.getClass());

		if (fields.isEmpty()) {
			GLog.error("no fields");
			throw new HORMException("no fields");
		}

		StringBuilder fds = new StringBuilder();
		StringBuilder fdsv = new StringBuilder();
		List<Object> params = new ArrayList<Object>();

		fds.append("(");
		fdsv.append("(");

		OptimisticLockInsert optimisticLockInsert = OptimisticLockWrapper.parseOptimisticLockInsert(t);

		try {

			if (optimisticLockInsert != null) {
				optimisticLockInsert.getVersionField().setAccessible(true);
				optimisticLockInsert.getVersionField().set(t, optimisticLockInsert.getInitVersionValue());
			}

			for (Field item : fields) {
				String fieldName = item.getName();
				if (map.containsKey(fieldName)) {
					item.setAccessible(true);
					Object v = item.get(t);
					// 值不为空才插入
					if (v != null) {
						String jdbcFieldName = map.get(fieldName);
						fds.append(escape0).append(jdbcFieldName).append(escape1).append(",");
						fdsv.append("?,");
						params.add(convertValue(v));
					}
				}
			}
		} catch (IllegalAccessException e) {
			throw new HORMException(e);
		}

		// 存在插入字段
		if (fds.length() > 1) {

			fds.deleteCharAt(fds.length() - 1);
			fdsv.deleteCharAt(fdsv.length() - 1);
			fds.append(")");
			fdsv.append(")");

		} else {

			GLog.error("no insert fields");
			throw new HORMException("no insert fields");
		}

		StringBuilder sb = new StringBuilder();
		sb.append("insert into ").append(escape0).append(tableName).append(escape1).append(fds).append(" values ")
				.append(fdsv);

		Object[] objects = new Object[params.size()];
		for (int i = 0; i < params.size(); ++i) {
			objects[i] = params.get(i);
		}

		String sql = sb.toString();

		GLog.debug("sql:{} , params:{}", sql, objects);
		return Pair.apply(sql, objects);
	}

	@Override
	public <T> Pair buildBatchInsertSQL(List<T> dataList) {

		if (dataList == null || dataList.isEmpty()) {
			throw new HORMException("batch data error");
		}

		Object presentObject = dataList.get(0);
		boolean addGmtCreateValue = presentObject instanceof BaseModel;

		Class<?> objectClass = presentObject.getClass();
		ModelFabric mf = ModelCacheUtil.getModelFabric(objectClass);
		String tableName = mf.getTableName();

		Map<String, Field> attrFields = mf.getFieldMap();
		Map<String, String> jdbcMap = mf.getJdbcMapping();

		List<String> columns = new ArrayList<>();
		Set<String> columnSet = jdbcMap.keySet();

		// 所有列
		if (OrmConfigHolder.isCaseSensitive()) {
			for (String c : columnSet) {
				columns.add(c);
			}
		} else {
			for (String c : columnSet) {
				String cupper = c.toUpperCase();
				if (!columns.contains(cupper)) {
					columns.add(cupper);
				}
			}
		}

		// 解析sql
		StringBuilder sb = new StringBuilder();
		sb.append("INSERT INTO ").append(escape0).append(tableName).append(escape1).append(" (");
		StringBuilder placeholders = new StringBuilder();
		for (int i = 0; i < columns.size(); ++i) {
			sb.append(escape0).append(columns.get(i)).append(escape1).append(",");
			placeholders.append("?,");
		}
		sb.deleteCharAt(sb.length() - 1);
		placeholders.deleteCharAt(placeholders.length() - 1);
		sb.append(") VALUES (").append(placeholders).append(")");
		String insertSql = sb.toString();

		// 所有值
		List<Object[]> params = new ArrayList<>();
		for (T t : dataList) {

			if (addGmtCreateValue) {
				((BaseModel) t).setGmtCreate(new Date());
			}

			Object[] row = new Object[columns.size()];

			for (int i = 0; i < columns.size(); ++i) {
				// jdbc 字段找到 attr
				String attr = jdbcMap.get(columns.get(i));
				// 通过 attr找到字段
				Field f = attrFields.get(attr);
				// 获取字段值
				row[i] = getObjectValue(f, t);
			}

			params.add(row);
		}

		GLog.debug("sql:{}", insertSql);

		return Pair.apply(insertSql, params);
	}

	private Object getObjectValue(Field f, Object t) {

		boolean preAccess = f.isAccessible();
		try {
			f.setAccessible(true);
			return convertValue(f.get(t));
		} catch (Exception e) {
			GLog.error("getObjectValue error" + e, e);
		} finally {
			f.setAccessible(preAccess);
		}

		return null;
	}

	@Override
	public <T> Pair buildUpdateSQL(T t) {

		if (t == null) {
			throw new HORMException("model is null");
		}

		ModelFabric mf = ModelCacheUtil.getModelFabric(t.getClass());
		Map<String, String> map = mf.getAttrMapping();
		List<String> pks = getPrimaryKey(mf.getPrimaryKey());
		String tableName = mf.getTableName();

		if (map == null || map.isEmpty()) {
			GLog.error("update mapper is empty");
			return null;
		}

		if (StrKit.isBlank(tableName)) {
			GLog.error("table name is empty");
			return null;
		}

		if (pks == null) {
			pks = Collections.emptyList();
		}

		// 获取所有字段
		List<Field> fields = ModelCacheUtil.getObjectAllFieldsWithcache(t.getClass());

		if (fields.isEmpty()) {
			return null;
		}

		// 主键
		Map<String, Object> pkKv = new HashMap<String, Object>();

		StringBuilder fds = new StringBuilder();
		List<Object> params = new ArrayList<Object>();

		// 获取可设置空字段值
		Set<String> nullableSets = new HashSet<String>();
		if (t instanceof Model) {
			nullableSets = ((Model) t).getNullableFields();
		}

		// 乐观锁信息
		OptimisticLockUpdate olu = OptimisticLockWrapper.parseOptimisticLockUpdate(t);

		try {

			if (olu != null) {
				olu.getVersionField().setAccessible(true);
				olu.getVersionField().set(t, olu.getNewVersionValue());
			}

			for (Field item : fields) {

				item.setAccessible(true);
				String fieldName = item.getName();

				// 判断主键
				if (pks.contains(fieldName)) {

					Object pkValue = item.get(t);
					String columnKey = map.get(fieldName);
					pkKv.put(columnKey, pkValue);

				} else {
					// 字段
					String f = map.get(fieldName);
					Object v = item.get(t);

					if (f != null) {// 找到字段

						boolean nullable = nullableSets.contains(fieldName);// 可为空

						if (v != null) {
							fds.append(escape0).append(f).append(escape1).append(" = ?,");
							params.add(convertValue(v));
						} else if (nullable) {
							// 可为空，则设置null值
							fds.append(escape0).append(f).append(escape1).append(" = NULL,");
						}

					}

				}
			}
		} catch (IllegalAccessException e) {
			throw new HORMException(e);
		}

		// 存在插入字段
		if (fds.length() > 0) {

			fds.deleteCharAt(fds.length() - 1);

		} else {

			throw new RuntimeException("update values is empty ");
		}

		StringBuilder sb = new StringBuilder();
		sb.append("update ").append(escape0).append(tableName).append(escape1).append(" set ").append(fds);

		if (!pkKv.isEmpty()) {

			final String token = " and ";

			sb.append(" where ");

			if (pks.size() == pkKv.size()) {
				// 全部匹配主键，按顺序组合

				for (String k : pks) {
					String c = map.get(k);
					sb.append(escape0).append(c).append(escape1).append(" = ?").append(token);
					params.add(pkKv.get(c));
				}

			} else {

				for (Entry<String, Object> entry : pkKv.entrySet()) {
					sb.append(escape0).append(entry.getKey()).append(escape1).append(" = ?").append(token);
					params.add(entry.getValue());
				}
			}

			// 此处加入乐观锁
			if (olu != null) {
				String columnKey = map.get(olu.getVersionFieldAttr());
				sb.append(escape0).append(columnKey).append(escape1).append(" = ?").append(token);
				params.add(olu.getOldVersionValue());
			}

			int totalLen = sb.length();
			int index = sb.length() - token.length();
			sb.delete(index, totalLen);
		}

		Object[] objects = new Object[params.size()];
		for (int i = 0; i < params.size(); ++i) {
			objects[i] = params.get(i);
		}

		String sql = sb.toString();
		GLog.debug("sql:{} , params:{}", sql, objects);

		return Pair.apply(sql, objects);
	}

	@Override
	public Pair buildDeleteByFieldSQL(List<Pair> kvList, Class<?> clz) {

		ModelFabric mf = ModelCacheUtil.getModelFabric(clz);

		String table = mf.getTableName();
		if (table == null) {
			throw new HORMException("对象没有Table注解");
		}

		Map<String, String> am = mf.getAttrMapping();
		if (am == null) {
			throw new HORMException("对象映射字段缺失");
		}

		String andToken = " and ";
		StringBuilder sb = new StringBuilder();
		List<Object> paramList = new ArrayList<>();

		for (int i = 0; i < kvList.size(); ++i) {
			Pair kv = kvList.get(i);
			String field = kv.getKey();
			Object value = kv.getValue();
			String trueField = am.get(field);
			if (trueField == null) {
				trueField = field;
			}
			if (value == null) {
				sb.append(escape0).append(trueField).append(escape1).append(" is null").append(andToken);
			} else {
				sb.append(escape0).append(trueField).append(escape1).append(" = ?").append(andToken);
				paramList.add(value);
			}

		}
		sb.delete(sb.length() - andToken.length(), sb.length());
		String sql = "delete from " + escape0 + table + escape1 + " where " + sb.toString();

		return Pair.apply(sql, paramList.toArray());
	}

	@Override
	public Pair buildQueryByFieldSQL(List<Pair> kvList, Class<?> clz) {

		return buildQueryByFieldSQL(kvList, clz, null);
	}

	@Override
	public Pair buildQueryByFieldSQL(List<Pair> kvList, Class<?> clz, String[] fields) {

		ModelFabric mf = ModelCacheUtil.getModelFabric(clz);

		String table = mf.getTableName();
		if (table == null) {
			throw new HORMException("对象没有Table注解");
		}

		Map<String, String> am = mf.getAttrMapping();
		if (am == null) {
			throw new HORMException("对象映射字段缺失");
		}

		String andToken = " and ";
		StringBuilder sb = new StringBuilder();

		List<Object> paramList = new ArrayList<>();

		for (int i = 0; i < kvList.size(); ++i) {
			Pair kv = kvList.get(i);

			String field = kv.getKey();
			Object value = kv.getValue();

			String trueField = am.get(field);
			if (trueField == null) {
				trueField = field;
			}
			if (value == null) {
				sb.append(escape0).append(trueField).append(escape1).append(" is null").append(andToken);
			} else {
				sb.append(escape0).append(trueField).append(escape1).append(" = ?").append(andToken);
				paramList.add(value);
			}

		}
		sb.delete(sb.length() - andToken.length(), sb.length());

		// 选择字段
		StringBuilder sf = new StringBuilder();
		if (fields != null && fields.length > 0) {
			for (String field : fields) {
				String trueField = am.get(field);
				if (trueField == null) {
					trueField = field;
				}
				sf.append(escape0).append(trueField).append(escape1).append(",");
			}
			sf.deleteCharAt(sf.length() - 1);
		} else {
			sf.append("*");
		}

		String sql = "select " + sf.toString() + " from " + escape0 + table + escape1 + " where " + sb.toString();

		return Pair.apply(sql, paramList.toArray());
	}

	@Override
	public String buildSimpleQuery(Class<?> clz) {
		return buildSimpleQuery(clz, null);
	}

	@Override
	public String buildSimpleQuery(Class<?> clz, String[] fields) {

		ModelFabric mf = ModelCacheUtil.getModelFabric(clz);
		String table = mf.getTableName();
		if (table == null) {
			throw new RuntimeException("对象没有Table注解");
		}

		Map<String, String> am = mf.getAttrMapping();
		if (fields == null || fields.length == 0 || am == null) {
			return "select * from " + escape0 + table + escape1;
		}

		StringBuilder sb = new StringBuilder("select ");
		for (String field : fields) {
			String trueField = am.get(field);
			if (trueField == null) {
				trueField = field;
			}
			sb.append(escape0).append(trueField).append(escape1).append(",");
		}
		sb.deleteCharAt(sb.length() - 1);
		sb.append(" from ").append(escape0).append(table).append(escape1);
		return sb.toString();
	}

	@Override
	public String buildValidateQuery() {
		return "SELECT 1";
	}

	@Override
	public String getTable(Class<?> clz) {
		return ModelCacheUtil.getModelFabric(clz).getTableName();
	}

	@Override
	public String getTableColumn(Class<?> clz, String prop) {
		ModelFabric mf = ModelCacheUtil.getModelFabric(clz);
		Map<String, String> attrMap = mf.getAttrMapping();
		String column = attrMap.get(prop);
		if (column == null) {
			column = prop;
		}
		return column;
	}

	@Override
	public String[] buildCreateTableDDL(Class<?> clz) {
		throw new UnsupportedOperationException("not implement");
	}

}
